# coding: utf-8
# pylint: disable=C0301,E1101

# Copyright 2017 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
ZDB, for matching short titles of journals.

See: goo.gl/90cteW
"""

import os

import luigi
from gluish.format import Gzip
from gluish.utils import shellout

from siskin.task import DefaultTask


class ZDBTask(DefaultTask):
    """
    ZDB base task.
    """
    TASK = 'zdb'


class ZDBDownload(ZDBTask):
    """
    Download snapshot.
    """
    format = luigi.Parameter(default="jsonld", description="rdf, hdt")

    def run(self):
        link = "http://datendienst.dnb.de/cgi-bin/mabit.pl?cmd=fetch&userID=opendata&pass=opendata&mabheft=ZDBTitel.%s.gz" % self.format
        output = shellout(""" wget -O {output} "{link}" """, link=link)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='jsonld.gz'), format=Gzip)


class ZDBShortTitleMap(ZDBTask):
    """
    Just an ad-hoc task with some ad-hoc code, refs #10562.
    """

    def requires(self):
        return ZDBDownload(format="rdf")

    def run(self):
        """
        @IHBhY2thZ2UgbWFpbgoK@IGltcG9ydCAoCiAg@@ImVuY29kaW5nL2pzb24iCiAg@@ImVuY29kaW5nL3htbCIK@@IC
        AibG9nIgog@@ICJvcyIKCiAg@@InN0cmluZ3MiCgog@@ICJnaXRodWIuY29tL21pa3UveG1sc3RyZWFtIgog@KQoK@
        IHR5cGUgRGVzY3JpcHRpb24gc3RydWN0IHsK@@ICBYTUxOYW1l@IHhtbC5OYW1lIGB4bWw6IkRlc2NyaXB0aW9uImA
        K@@ICBJc3Nu@@IFtdc3RyaW5nIGB4bWw6Imlzc24iYAog@@IFNob3J0VGl0bGUgW11zdHJpbmcgYHhtbDoic2hvcnR
        UaXRsZSJgCiAg@@VGl0bGUg@ICBbXXN0cmluZyBgeG1sOiJ0aXRsZSJgCiAgICB9Cgog@ZnVuYyBtYWluKCkgewog@
        @IHNtIDo9IG1ha2UobWFwW3N0cmluZ11zdHJpbmcpCiAg@@c2Nhbm5lciA6PSB4bWxzdHJlYW0uTmV3U2Nhbm5lcih
        vcy5TdGRpbiwgbmV3KERlc2NyaXB0aW9uKSkK@@ICBmb3Igc2Nhbm5lci5TY2FuKCkgewog@@@ICB0YWcgOj0gc2Nh
        bm5lci5FbGVtZW50KCkK@@@@aWYgdiwgb2sgOj0gdGFnLigqRGVzY3JpcHRpb24pOyBvayB7CiAg@@@@ICBpZiBsZW
        4odi5TaG9ydFRpdGxlKSA9PSAwIHsK@@@@@@ICBjb250aW51ZQog@@@@@fQog@@@@@Zm9yIF8sIHMgOj0gcmFuZ2Ug
        di5TaG9ydFRpdGxlIHsK@@@@@@ICBmb3IgXywgdCA6PSByYW5nZSB2LlRpdGxlIHsK@@@@@@@@c21bc10gPSB0CiAg
        @@@@@@@IHNtW3N0cmluZ3MuVG9Mb3dlcihzKV0gPSB0CiAg@@@@@@fQog@@@@@fQog@@@ICB9CiAg@@fQog@@IGlmI
        GVyciA6PSBzY2FubmVyLkVycigpOyBlcnIgIT0gbmlsIHsK@@@@bG9nLkZhdGFsKGVycikK@@ICB9CiAg@@aWYgZXJ
        yIDo9IGpzb24uTmV3RW5jb2Rlcihvcy5TdGRvdXQpLkVuY29kZShzbSk7IGVyciAhPSBuaWwgewog@@@ICBsb2cuRm
        F0YWwoZXJyKQog@@IH0K@IH0K
        """
        source = self.run.__doc__.replace(
            "\n", "").replace(" ", "").replace("@", "ICAg")
        tempcode = shellout("""echo '{code}' | base64 -d > {output}.go """,
                            code=source, preserve_whitespace=True)
        output = shellout(""" unpigz -c {input} | go run {code}.go > {output} """,
                          code=tempcode, input=self.input().path)
        os.remove(tempcode)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='json'))



#!/usr/bin/env python3
# coding: utf-8
# pylint: disable=C0103

from __future__ import print_function

import base64
import io
import os
import re
import sqlite3
import sys
import tempfile
from builtins import bytes

import marcx
import xmltodict


def clear_format(format):
    if isinstance(format, list):
        format = format[0]
    format = format.strip()
    format = format.rstrip(";")
    format = format.lower()
    format = format.split("; ") # manchmal sind mehrere Formate angegeben
    format = format[0] # nur das zuerst angegebene Format wird genommen
    return format

def get_leader(format='photograph'):
    return "     %s  22        4500" % formatmaps[format]["leader"]

def get_field_008(format='photograph', f041_a="   ", f260_c="20uu"):
    if "008" not in formatmaps[format]:
        return "130227u%suuuuxx uuup%s  c" % (f260_c, f041_a)
    return "130227%s%suuuuxx uuup%s  c" % (formatmaps[format]["008"], f260_c, f041_a)

def get_field_935b(format='photograph'):
    if "935b" not in formatmaps[format]:
        return ""
    return formatmaps[format]["935b"]

langmap = {
    "English": "eng",
    "German": "ger",
    "French": "fre",
    "Italian": "ita",
    "Russian": "rus",
    "Spanish": "spa"
    }

formatmaps = {
    'periodical':
    {
        '008': 'n',
        'leader': 'nas',
        '935b' : 'cofz'
    },
    'ephemera':
    {
        'leader': 'ckm',
        '935b' : 'foto'
    },
    'magazine cover':
    {
        'leader': 'ckm',
        '935b' : 'foto'
    },
    'photograph':
    {
        'leader': 'ckm',
        '935b' : 'foto'
    },
    'postcard':
    {
        'leader': 'ckm',
        '935b' : 'foto'
    },
    'clipping':
    {
        'leader': 'ckm',
        '935b' : 'foto'
    },
    'flier (printed matter)':
    {
        'leader': 'nam'
    },
    'storyboard':
    {
        'leader': 'nam'
    },
    'sheet music':
    {
        'leader': 'ncs'
    },
    'correspondence':
    {
        'leader': 'nam'
    },
    'pamphlet':
    {
        'leader': 'nam'
    },
    'correspondence document':
    {
        '007': 'kf',
        'leader': 'nam'
    },
    'lobby card':
    {
        'leader': 'ckm',
        '935b' : 'foto'
    },
    'photomechanical print':
    {
        'leader': 'nam'
    },
    'cigarette cards':
    {
        'leader': 'nam'
    },
    'souvenir handkerchief box':
    {
        'leader': 'ckm',
        '935b' : 'foto'
    },
    'scrapbook':
    {
        'leader': 'nam',
        '935b' : 'cofz'
    },
    'slides (photographs)':
    {
        'leader': 'ckm',
        '935b' : 'foto'
    },
    'program (document)':
    {
        'leader': 'nam',
        '935b' : 'cofz'
    },
    'pressbook':
    {
        'leader': 'nam',
        '935b' : 'cofz'
    },
    'autograph album':
    {
        'leader': 'ntm',
        '935b' : 'handschr'
    },
    'monograph':
    {
        'leader': 'nam',
        '935b' : 'cofz'
    },
    'drawing':
    {
        'leader': 'nam'
    },
}

if len(sys.argv) < 3:
    print('usage: %s INFILE OUTFILE' % sys.argv[0], file=sys.stderr)
    sys.exit(1)

_, dbfile = tempfile.mkstemp(prefix='103-')

sqlitecon = sqlite3.connect(dbfile)
sqlite = sqlitecon.cursor()
sqlite.execute("DROP TABLE IF EXISTS record")

query = """
    CREATE TABLE
       'record' ('id' INTEGER PRIMARY KEY AUTOINCREMENT,
                 'identifier' TEXT,
                 'title' TEXT,
                 'description' TEXT,
                 'subject' TEXT,
                 'type' TEXT,
                 'format' TEXT,
                 'relation' TEXT,
                 'publisher' TEXT,
                 'date' TEXT,
                 'source' TEXT,
                 'language' TEXT,
                 'rights' TEXT,
                 'url' TEXT,
                 'creator' TEXT,
                 'coverage' TEXT)
"""

sqlite.execute(query)

inputfile = io.open(sys.argv[1], "r", encoding="utf-8")
outputfile = io.open(sys.argv[2], "wb")

xml = xmltodict.parse(inputfile.read())

identifier = ""
title = ""
description = ""
subject = ""
type = ""
format = ""
relation = ""
publisher = ""
date = ""
source = ""
language = ""
rights = ""
url = ""
creator = ""
coverage = ""

for record in xml["Records"]["Record"]:

    if record["header"]["@status"] != "deleted":

        identifier = record["header"]["identifier"]

        title = record.get("metadata").get("oai_dc:dc").get("dc:title", "")
        if isinstance(title, list):
            title = "||".join(title)

        description = record.get("metadata").get("oai_dc:dc").get("dc:description", "")
        if isinstance(description, list):
            description = "||".join(description)

        subject = record.get("metadata").get("oai_dc:dc").get("dc:subject", "")
        if isinstance(subject, list):
            subject = "; ".join(subject)

        type = record.get("metadata").get("oai_dc:dc").get("dc:type", "")
        if isinstance(type, list):
            type = "||".join(type)

        format = record.get("metadata").get("oai_dc:dc").get("dc:format", "")
        if isinstance(format, list):
            format = "||".join(format)

        relation = record.get("metadata").get("oai_dc:dc").get("dc:relation", "")
        if isinstance(relation, list):
            relation = "||".join(relation)

        publisher = record.get("metadata").get("oai_dc:dc").get("dc:publisher", "")
        if isinstance(publisher, list):
            publisher = "||".join(publisher)

        date = record.get("metadata").get("oai_dc:dc").get("dc:date", "")
        if isinstance(date, list):
            date = "||".join(date)

        source = record.get("metadata").get("oai_dc:dc").get("dc:source", "")
        if isinstance(source, list):
            source = source[0]

        language = record.get("metadata").get("oai_dc:dc").get("dc:language", "")
        if isinstance(language, list):
            language = "||".join(language)

        rights = record.get("metadata").get("oai_dc:dc").get("dc:rights", "")
        if isinstance(rights, list):
            rights = "||".join(rights)

        url = record.get("metadata").get("oai_dc:dc").get("dc:identifier", "")
        if isinstance(url, list):
            url = url[1]

        creator = record.get("metadata").get("oai_dc:dc").get("dc:creator", "")
        if isinstance(creator, list):
            creator = "||".join(creator)

        coverage = record.get("metadata").get("oai_dc:dc").get("dc:coverage", "")
        if isinstance(coverage, list):
            coverage = "||".join(coverage)

        values = (title, format, type)

        query = """
            SELECT
                id,
                url,
                COUNT(id) AS matches
            FROM
                record
            WHERE
                title = ? AND
                format = ? AND
                type = ?
        """

        sqlite.execute(query, values)
        row = sqlite.fetchone()

        id = row[0]
        url_old = row[1]
        matches = row[2]

        if matches == 0:

            values = (identifier, title, description, subject, type, format, relation, publisher, date, source, language, rights, url, creator, coverage)

            query = """
                INSERT INTO
                    record (identifier,
                            title,
                            description,
                            subject,
                            type,
                            format,
                            relation,
                            publisher,
                            date,
                            source,
                            language,
                            rights,
                            url,
                            creator,
                            coverage)
                VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
            """
        else:

            url = url_old + "||" + url
            values = (url, id)

            query = """
                UPDATE
                    record
                SET
                    url = ?
                WHERE
                    id = ?
            """

        sqlite.execute(query, values)

        identifier = ""
        title = ""
        description = ""
        subject = ""
        type = ""
        format = ""
        relation = ""
        publisher = ""
        date = ""
        source = ""
        language = ""
        rights = ""
        url = ""
        creator = ""
        coverage = ""

query = "SELECT * FROM record"
sqlite.execute(query)
rows = sqlite.fetchall()

for row in rows:
    identifier = row[1]
    identifier = bytes(identifier, "utf-8")
    identifier = base64.b64encode(identifier)
    identifier = identifier.decode("utf-8").rstrip("=")
    title = row[2]
    description = row[3]
    subject = row[4]
    type = row[5]
    format = row[6]
    format = clear_format(format)
    relation = row[7]
    publisher = row[8]
    date = row[9]
    source = row[10]
    language = row[11]
    rights = row[12]
    url = row[13]
    creator = row[14]
    coverage = row[15]

    if identifier == "":
        continue

    if format not in formatmaps:
        print("Das Format %s fehlt in der Lookup-Tabelle. Default-Wert ist 'photograph'" % format, file=sys.stderr)
        format = "photograph"

    f001 = "finc-103-%s" % identifier
    leader = get_leader(format=format)

    if language != "":
        language = language.rstrip(";") # manchmal endet die Sprache auf ";"
        language = language.replace(" ", "") # manchmal gibt es Leerzeichen mittendrin oder am Ende
        language = language.split(";") # manchmal sind mehrere Sprache angegeben
        if isinstance(language, list):
            language = language[0] # nur die zuerst angegebene Sprache wird berücksichtig

        f041_a = langmap.get(language, "")

        if f041_a == "":
            print("Die Sprache %s fehlt in der Lookup-Tabelle!" % language)
    else:
        f041_a = ""

    f008 = get_field_008(format=format, f041_a=f041_a)

    if creator != "":
        creator = creator.split(" ; ")
        f100_a = creator[0]
        if len(creator) > 1:
            f700_a = []
            for creator_part in creator[1:]:
                f700_a.append(creator_part)
        else:
            f700_a = ""
    else:
        f100_a = ""
        f700_a = ""

    if title != "":
        f245_a = title
    else:
        f245_a = ""

    # Chicago : Selig Polyscope Co.
    # Shurey's Publications
    if publisher != "":
        match = re.search("(.*)\s:\s(.*)", publisher)
        if match:
            f260_a, f260_b = match.groups()
            f260_b = " : %s" % f260_b
        else:
            f260_a = ""
            f260_b = publisher
    else:
        f260_b = ""
        f260_a = ""

    # mögliche Muster: 1932-04-01 | 1930; 1931 | 1922
    if date != "":
        match = re.search("(^\d\d\d\d)", date)
        if match:
            f260_c = match.group(1)
            if publisher != "":
                f260_b = f260_b + ", "
        else:
            f260_c = ""
    else:
        f260_c = ""

    if source != "":
        f490_a = source
    else:
        f490_a = ""

    f500_a = []
    if rights != "":
        f500_a.append(rights)

    if coverage != "":
        f500_a.append(coverage)

    if rights == "" and coverage == "":
        f500_a = ""

    if description != "":
        description = description.split("||")
        f520_a = []
        for description_part in description:
            f520_a.append(description_part)
    else:
        f520_a = ""

    if subject != "" or format != "":
        # manchmal ended die Schlagwortfolgfe auf ;
        subject = subject.rstrip(";")

        # die Trennzeichen zwischen den Schlagwörtwern sind in der Quelle uneinheitlich: mal " ; " mal "; " mal " ;"
        subject = subject.replace(" ; ", ";")
        subject = subject.replace(" ;", ";")
        subject = subject.replace("; ", ";")
        subject = subject.split(";")

        f689_a = []
        for subject_part in subject:
            f689_a.append(subject_part)

        f689_a.append(format.title())

    else:
        f689_a = ""

    if url != "":
        url = url.split("||")
        f856_u = set(url)
        if len(f856_u) > 1:
            items = len(f856_u)
            f520_a = "Contains a collection of %s Pictures." % items

    #f935_b = get_field_935b(format=format)
    f980_a = identifier
    f980_b = "103"

    marcrecord = marcx.Record(force_utf8=True)
    marcrecord.strict = False
    marcrecord.leader = leader
    marcrecord.add("001", data=f001)
    marcrecord.add("007", data="cr")
    f008 = get_field_008(format=format, f041_a=f041_a, f260_c=f260_c)
    marcrecord.add("008", data=f008)
    marcrecord.add("041", a=f041_a)
    marcrecord.add("100", a=f100_a)
    marcrecord.add("245", a=f245_a)
    publisher = ["a", f260_a, "b", f260_b, "c", f260_c]
    marcrecord.add("260", subfields=publisher)
    marcrecord.add("490", a=f490_a)

    for value in f500_a:
        marcrecord.add("500", a=value)

    if isinstance(f520_a, list):
        for value in f520_a:
            marcrecord.add("520", a=value)
    else:
        marcrecord.add("520", a=f520_a)

    for value in f689_a:
        marcrecord.add("689", a=value)

    for value in f700_a:
        marcrecord.add("700", a=value)

    if len(f856_u) > 1:
        for i, url in enumerate(f856_u, start=1):
            marcrecord.add("856", q="text/html", _3="Picture %s" % i, u=url)
    else:
        marcrecord.add("856", q="text/html", _3="Link zur Ressource", u=f856_u)

    #marcrecord.add("935", b=f935_b)
    marcrecord.add("980", a=f980_a, b=f980_b, c="sid-103-col-margaret")
    outputfile.write(marcrecord.as_marc())

sqlitecon.commit()
sqlitecon.close()
inputfile.close()
outputfile.close()
os.remove(dbfile)



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,R0904,E1103,C0301

# Copyright 2018 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Robert Schenk, <schenk@ub.uni-leipzig.de>
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Music Treasures Consortium Online, refs #5798.
"""

from __future__ import print_function

import datetime
import json
import sys
import time

import luigi
import requests
from gluish.utils import shellout

from siskin.task import DefaultTask


class MTCTask(DefaultTask):
    """
    Base task for MTC.
    """
    TAG = '10'


class MTCHarvest(MTCTask):
    """
    Harvest.
    """
    max_retries = luigi.IntParameter(default=3, significant=False)

    def run(self):
        page, retry_count = 1, self.max_retries
        headers = {'user-agent': 'Mozilla/5.0 (Windows NT x.y; rv:10.0) Gecko/20100101 Firefox/10.0'}

        with self.output().open("w") as output:
            while True:
                url = "https://www.loc.gov/collections/music-treasures-consortium/?sp=%s&fo=json" % page
                self.logger.debug(url)
                r = requests.get(url, headers=headers)
                if r.status_code >= 500:
                    if retry_count == 0:
                        raise RuntimeError("failed after %s attempts with %s: %s",
                                           self.max_retries, r.status_code, url)
                    else:
                        self.logger.debug("retrying %s, got %s", url, r.status_code)
                        retry_count -= 1
                        continue

                doc = json.loads(r.text)
                if doc.get("status") == 404:
                    self.logger.debug("404 at %s", url)
                    break
                output.write(r.text)
                output.write("\n")

                page += 1
                retry_count = self.max_retries
                time.sleep(5)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='json'))


class MTCMARC(MTCTask):
    """
    Convert custom JSON.
    """

    def requires(self):
        return MTCHarvest()

    def run(self):
        output = shellout("python {script} {input} {output}",
                          script=self.assets('10/10_marcbinary.py'),
                          input=self.input().path)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='fincmarc.mrc'))



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103,C0301

# Copyright 2017 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Robert Schenk, <robert.schenk@uni-leipzig.de>
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Egyptology at Leipzig University, refs #5246.

This source only contains an ExternalTask, pointing to a file as configured.
This file has been created from an sqlite3 dump through a couple of steps, some
of them not documented in siskin/assets/70.

[egyptology]

file = /path/to/output.fincmarc.xml
"""

import luigi

from siskin.task import DefaultTask


class EgyptologyTask(DefaultTask):
    """ Base task. """
    TAG = '70'


class EgyptologyFincMARC(EgyptologyTask, luigi.ExternalTask):
    """
    Point to final result.
    """

    def output(self):
        return luigi.LocalTarget(path=self.config.get('egyptology', 'file'))



# coding: utf-8
# pylint: disable=C0301,E1101

# Copyright 2016 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Robert Schenk, <robert.schenk@uni-leipzig.de>
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Gutenberg
---------

https://www.gutenberg.org/, refs #10875, #5520.

"""

import datetime

import luigi
from gluish.intervals import monthly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout

from siskin.task import DefaultTask


class GutenbergTask(DefaultTask):
    """
    Base task for Gutenberg.
    """
    TAG = '1'

    def closest(self):
        return monthly(date=self.date)


class GutenbergDownload(GutenbergTask):
    """
    Download RDF dump.

    TODO(miku): Allow for http_proxy, as still: "Your IP Address in Germany is
    Blocked from www.gutenberg.org" - A Court in Germany ordered that access to
    certain items in the Project Gutenberg collection are blocked from Germany.
    Project Gutenberg believes the Court has no jurisdiction over the matter,
    but until the issue is resolved, it will comply.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def run(self):
        output = shellout("wget -O {output} http://www.gutenberg.org/cache/epub/feeds/rdf-files.tar.zip")
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='tar.zip'))


class GutenbergMARC(GutenbergTask):
    """
    Extract and convert.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return GutenbergDownload(date=self.date)

    def run(self):
        output = shellout("unzip -p {input} | tar -xO | python {script} - > {output}",
                          input=self.input().path, script=self.assets('1/1_marcbinary.py'))
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='fincmarc.mrc'))



# coding: utf-8
# pylint: disable=C0301,E1101

# Copyright 2015 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
arxiv.org, https://arxiv.org/help/oa/index

Config:

[core]

metha-dir = /path/to/dir

----

From: https://groups.google.com/forum/#!topic/arxiv-api/aOacIt6KD2E

> the problem is that the API is not the appropriate vehicle to obtain a full
copy of arXiv metadata. It is intended for interactive queries and search
overlays and should not be used for full replication. The size of result sets
is capped at a large but finite number. You should use OAI-PMH instead for your
purposes.

* https://academia.stackexchange.com/q/38969/6198

"""

import datetime

import luigi
from gluish.common import Executable
from gluish.intervals import weekly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout

from siskin.task import DefaultTask


class ArxivTask(DefaultTask):
    """
    Base task.
    """
    TAG = '121'

    def closest(self):
        return weekly(date=self.date)


class ArxivCombine(ArxivTask):
    """
    Single file dump.
    """
    date = ClosestDateParameter(default=datetime.date.today())
    url = luigi.Parameter(default="http://export.arxiv.org/oai2", significant=False)
    prefix = luigi.Parameter(default="oai_dc", significant=False)

    def requires(self):
        return Executable(name='metha-sync', message='https://github.com/miku/metha')

    def run(self):
        shellout("METHA_DIR={dir} metha-sync -format {prefix} {url}",
                 prefix=self.prefix, url=self.url, dir=self.config.get('core', 'metha-dir'))
        output = shellout("METHA_DIR={dir} metha-cat -root Records -format {prefix} {url} | pigz -c > {output}",
                          prefix=self.prefix, url=self.url, dir=self.config.get('core', 'metha-dir'))
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext="xml.gz"))


class ArxivIntermediateSchema(ArxivTask):
    """
    Better, still experimental.

    Convert to intermediate schema via metafacture. Custom morphs and flux are
    kept in assets/arxiv. Maps are kept in assets/maps.

    Note: There are early arxiv XML records, that do not get an ID in the transformation, e.g.

    {
      "finc.mega_collection": "Arxiv",
      "finc.source_id": "arxiv",
      "rft.atitle": "A determinant of Stirling cycle numbers counts unlabeled acyclic   single-source automata",
      "abstract": "Comment: 11 pages",
      "rft.date": "2007-03-30",
      "x.date": "2007-03-30T00:00:00Z",
      "authors": [
        {
          "rft.au": "Callan, David"
        }
      ],
      "url": [
        "http://arxiv.org/abs/0704.0004"
      ],
      "finc.format": "text",
      "x.subjects": [
        "Mathematics - Combinatorics",
        "05A15"
      ]
    }

    """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return ArxivCombine(date=self.date)

    def run(self):
        """
        With 3.1.1 -Xmx512m seemed to be enough. Also: java -Xmx1G -XX:+PrintFlagsFinal -Xmx2G 2> /dev/null | grep MaxHeapSize.
        """
        mapdir = 'file:///%s' % self.assets("maps/")
        output = shellout("""FLUX_JAVA_OPTIONS="-Xmx2048m" flux.sh {flux} in={input} MAP_DIR={mapdir} | pigz -c > {output}""",
                          flux=self.assets("arxiv/arxiv.flux"), mapdir=mapdir, input=self.input().path)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj.gz'))




# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,R0904,E1103,C0301

# Copyright 2018 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Genderopen OAI.
"""

import datetime

import luigi
from gluish.utils import shellout
from gluish.parameter import ClosestDateParameter
from gluish.format import Gzip
from gluish.intervals import monthly

from siskin.task import DefaultTask


class GenderopenTask(DefaultTask):
    """
    Base task for gender open.
    """
    TAG = '162'
    REPO = "https://www.genderopen.de/oai/request"

    def closest(self):
        return monthly(self.date)


class GenderopenIntermediateSchema(GenderopenTask):
    """
    Harvest and convert.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def run(self):
        output = shellout("metha-sync {repo} && metha-cat {repo} | span-import -i genderopen | gzip -c > {output}",
                          repo=self.REPO)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj.gz'), format=Gzip)



#!/usr/bin/env python
# coding: utf-8

from builtins import *

from io import StringIO, BytesIO
import io
import re
import sys
import pymarc

import marcx
from siskin.utils import xmlstream


copytags = ["003", "005", "007", "008", "022", "024", "040", "041", "100", "102",
            "108", "110", "112", "115", "120", "123", "124", "130", "137", "139", "140",
            "144", "146", "150", "151", "152", "159", "160", "166", "174", "175", "177",
            "178", "179", "180", "181", "182", "183", "184", "185", "186", "187", "189",
            "190", "191", "192", "193", "194", "195", "196", "197", "198", "199", "200",
            "206", "211", "236", "256", "260", "326", "363", "370", "377", "391", "411",
            "501", "506", "520", "542", "550", "600", "604", "650", "653", "690", "700",
            "710", "742", "760", "773", "777", "787", "797", "856", "860", "873",
            "880", "947", "948", "961"]


inputfilename = "39_input.xml"
outputfilename = "39_output.mrc"
filterfilename = "39_issn_filter"

if len(sys.argv) >= 4:
    inputfilename, outputfilename, filterfilename = sys.argv[1:4]

outputfile = io.open(outputfilename, "wb")
filterfile = io.open(filterfilename, "r")

issn_list = filterfile.readlines()
issn_list = [issn.rstrip("\n") for issn in issn_list]

#for oldrecord in reader:

for oldrecord in xmlstream(inputfilename, "record"):
    
    oldrecord = BytesIO(oldrecord)
    oldrecord = pymarc.marcxml.parse_xml_to_array(oldrecord)
    oldrecord = oldrecord[0]
    newrecord = marcx.Record(force_utf8=True)
    newrecord.strict = False
    
    # prüfen, ob Titel vorhanden ist
    if not oldrecord["245"]:
        continue

    # Leader
    newrecord.leader = "     " + oldrecord.leader[5:]

    # Identifikator   
    f001 = oldrecord["001"].data
    f001 = f001.replace("-", "").replace("_", "")
    newrecord.add("001", data="finc-39-%s" % f001)
    
    # Originalfelder, die ohne Änderung übernommen werden
    for tag in copytags:
        for field in oldrecord.get_fields(tag):
            newrecord.add_field(field)

    # Titel
    # Wird extra verarbeitet, da Unterfelder teilweise leer sind und dann nicht angelegt werden sollen
    try:
        f245a = oldrecord["245"]["a"]
    except:
        f245a = ""

    try:
        f245b = oldrecord["245"]["b"]
    except:
        f245b = ""

    try:
        f245c = oldrecord["245"]["c"]
    except:
        f245c = ""

    try:
        f245h = oldrecord["245"]["h"]
    except:
        f245h = ""

    subfields = ["a", f245a, "b", f245b, "c", f245c, "h", f245h]
    newrecord.add("245", subfields=subfields)     

    # Kollektion und Ansigelung
    try:
        f022a = oldrecord["022"]["a"]
    except:
        f022a = ""

    try:
        f760x = oldrecord["760"]["x"]
    except:
        f760x = ""

    try:
         f787x = oldrecord["787"]["x"]
    except:
        f787x = ""      

    if f022a in issn_list or f760x in issn_list or f787x in issn_list:
        collections = ["a", f001, "b", "39", "c", u"sid-39-col-persee", "c", u"sid-39-col-perseeadlr"]
    else:
        collections = ["a", f001, "b", "39", "c", u"sid-39-col-persee"]

    newrecord.add("980", subfields=collections)
 
    outputfile.write(newrecord.as_marc())

outputfile.close()
filterfile.close()



# coding: utf-8
# pylint: disable=C0301,E1101

# Copyright 2015 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

import datetime

import luigi
from gluish.intervals import monthly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout

from siskin.task import DefaultTask


class DBLPTask(DefaultTask):
    """
    DBLP is a computer science bibliography website. Starting in 1993 at the
    University of Trier, Germany, it grew from a small collection of HTML
    files[1] and became an organization hosting a database and logic
    programming bibliography site. DBLP listed more than 3.4 million journal
    articles, conference papers, and other publications on computer science in
    July 2016, up from about 14,000 in 1995.[2] All important journals on
    computer science are tracked. Proceedings papers of many conferences are
    also tracked. It is mirrored at three sites across the Internet
    """
    TAG = 'dblp'

    def closest(self):
        return monthly(date=self.date)


class DBLPDownload(DBLPTask):
    """ Download file. """
    url = luigi.Parameter(default='http://dblp.uni-trier.de/xml/dblp.xml.gz', significant=False)
    date = ClosestDateParameter(default=datetime.date.today())

    def run(self):
        output = shellout("curl --fail -L {url} > {output}", url=self.url)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml.gz'))


class DBLPDOIList(DBLPTask):
    """ QnD doi list. """

    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return DBLPDownload()

    def run(self):
        output = shellout("""LC_ALL=C grep "doi.org" <(unpigz -c {input}) |
                             LC_ALL=C sed -e 's@<ee>https://doi.org/@@g;s@</ee>@@g' |
                             LC_ALL=C grep ^10 |
                             LC_ALL=C sort -S50% > {output}""", input=self.input().path)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path())



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103,C0301,C0103,W0614,W0401,E0202

# Copyright 2015 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Various utilities.
"""

from __future__ import print_function

import errno
import hashlib
import itertools
import json
import logging
import operator
import os
import random
import re
import string
import sys
import tempfile
import xml.etree.cElementTree as ET

import requests
import six
from six.moves.urllib.parse import urlparse

import backoff
import bs4
import luigi
from dateutil import relativedelta
from future import standard_library
from siskin import __version__

# XXX: move to six.
standard_library.install_aliases()

logger = logging.getLogger('siskin')

class SetEncoder(json.JSONEncoder):
    """
    Helper to encode python sets into JSON lists.

    So you can write something like this:

        json.dumps({"things": set([1, 2, 3], cls=SetEncoder)}
    """
    def default(self, obj):
        """
        Decorate call to standard implementation.
        """
        if isinstance(obj, set):
            return list(obj)
        return json.JSONEncoder.default(self, obj)

def date_range(start_date, end_date, increment, period):
    raise NotImplementedError('use: from gluish.utils import date_range, https://git.io/fpDU1')

def iterfiles(directory='.', fun=None):
    """
    Shortcut for os.walk, yield paths below a directory, optionally filter the
    paths by function given in `fun`.
    """
    if fun is None:
        fun = lambda path: True
    for root, _, files in os.walk(directory):
        for f in files:
            path = os.path.join(root, f)
            if fun(path):
                yield path

def random_string(length=16):
    """
    Return a random string(upper and lowercase letters) of length `length`,
    defaults to 16.
    """
    return ''.join(random.choice(string.ascii_letters) for _ in range(length))

def nwise(iterable, n=2):
    """
    Generalized: func: `pairwise`. Split an iterable after every `n` items.
    """
    i = iter(iterable)
    piece = tuple(itertools.islice(i, n))
    while piece:
        yield piece
        piece = tuple(itertools.islice(i, n))

def dictcheck(obj, contains=None, absent=None, ignore=None):
    """
    Check if a dictionary contains values for the keys given in `contains` and at
    the same time it does not contain values for keys, which are given in
    `absent`.

    >>> dictcheck({"name": "x"}, contains=["name"])
    True

    >>> dictcheck({"name": "x"}, contains=["name"], absent=["key"])
    True

    >>> dictcheck({"name": "x", "key": "1234"}, contains=["name"], absent=["key"])
    False

    >>> dictcheck({"key": None}, absent=["key"])
    True

    >>> dictcheck({}, absent=["key"])
    True

    This is a helper for AMSL-API cases (https://git.io/vpHgS).
    """
    if not isinstance(obj, dict):
        raise ValueError('dictionary only')
    if contains is None:
        contains = []
    if absent is None:
        absent = []
    if ignore is None:
        ignore = []

    for key in contains:
        if key in ignore:
            continue
        if key not in obj:
            return False
        if not bool(operator.itemgetter(key)(obj)):
            return False

    for key in absent:
        if key in ignore:
            continue
        if key not in obj:
            continue
        if bool(operator.itemgetter(key)(obj)):
            return False

    return True

def get_task_import_cache():
    """
    Load or create `taskname: modulename` mappings. Return a tuple containing
    the dictionary and the path to the cache file.

    The command line entry points (e.g. taskdo and friends) need to import all
    modules in order to find a task. The import process actually takes a while
    (few seconds), so the import cache shortens the startup time of the command
    line tools by only importing the module the given task is in.

    It is save to remove the file returned by `taskimportcache` at any time.
    """
    task_import_cache = None
    path = os.path.join(tempfile.gettempdir(),
                        'siskin_task_import_cache_%s' % __version__)
    if not os.path.exists(path):
        logger.debug("creating task import cache at %s", path)
        from siskin.cacheutils import _write_task_import_cache
        _write_task_import_cache(path)

    with open(path) as handle:
        try:
            task_import_cache = json.load(handle)
        except Exception as err:
            print("failed to load task import cache, remove %s, then try again" %
                  path, file=sys.stderr)
            sys.exit(1)

    return task_import_cache, path

def load_set_from_target(target):
    """
    Given a luigi.LocalTarget, load each line of the file into a set.
    """
    s = set()
    with target.open() as handle:
        for line in handle:
            s.add(line.strip())
    return s

def load_set_from_file(filename, func=lambda v: v):
    """
    Given a filename, load each non-empty line into a set.
    """
    s = set()
    with open(filename) as handle:
        for line in (str.strip(line) for line in handle):
            if not line:
                continue
            s.add(func(line))
    return s

class URLCache(object):
    """
    A simple URL content cache. Stores everything on the filesystem. Content
    is first written to a temporary file and then renamed. With concurrent
    requests for the same URL, the last one wins. Raises exception on any HTTP
    status >= 400. Retries supported.

    It is not very efficient, as it creates lots of directories.
    > 396140 directories, 334024 files ... ...

    To clean the cache just remove the cache directory.

    >>> cache = URLCache()
    >>> cache.get_cache_file("https://www.google.com")
    /tmp/ef/7e/fc/ef7efc9839c3ee036f023e9635bc3b056d6ee2d

    >>> cache.is_cached("https://www.google.com")
    False

    >>> page = cache.get("https://www.google.com")
    >>> page[:15]
    '<!doctype html>'

    >>> cache.is_cached("https://www.google.com")
    True

    It is possible to force a download, too:

    >>> page = cache.get("https://www.google.com", force=True)
    """

    def __init__(self, directory=None, max_tries=12):
        """
        If `directory` is not explictly given, all files will be stored under
        the temporary directory. Requests can be retried, if they resulted in
        a non 200 HTTP status code. A server might send a HTTP 500 (Internal
        Server Error), even if it really is a HTTP 503 (Service Unavailable).
        We therefore treat HTTP 500 errors as something to retry on,
        at most `max_tries` times.
        """
        self.directory = directory or tempfile.gettempdir()
        self.sess = requests.session()
        self.max_tries = max_tries

    def get_cache_file(self, url):
        """
        Return the cache file path for a URL. This will - as a side effect -
        create the parent directories, if necessary.
        """
        digest = hashlib.sha1(six.b(url)).hexdigest()
        d0, d1, d2 = digest[:2], digest[2:4], digest[4:6]
        path = os.path.join(self.directory, d0, d1, d2)

        if not os.path.exists(path):
            try:
                os.makedirs(path)
            except OSError as e:
                if e.errno == errno.EEXIST:
                    pass
                else:
                    raise
        return os.path.join(path, digest)

    def is_cached(self, url):
        return os.path.exists(self.get_cache_file(url))

    def get(self, url, force=False):
        """
        Return URL, either from cache or the web.
        """

        @backoff.on_exception(backoff.expo, RuntimeError, max_tries=self.max_tries)
        def fetch(url):
            """
            Nested function, so we can configure number of retries.
            """
            r = self.sess.get(url, timeout=600)
            if r.status_code >= 400:
                raise RuntimeError('%s on %s' % (r.status_code, url))
            with tempfile.NamedTemporaryFile(delete=False) as output:
                output.write(r.text.encode('utf-8'))
            os.rename(output.name, self.get_cache_file(url))

        if not self.is_cached(url) or force is True:
            fetch(url)

        with open(self.get_cache_file(url)) as handle:
            return handle.read()

def scrape_html_listing(url, with_head=False):
    """
    Given a URL to a webpage containing a simple file listing, try to return a
    list of links to the files on the page.

    >>> scrape_html_listing("https://ftp.halifax.rwth-aachen.de/archlinux/iso/2018.12.01/")
    ['https://ftp.halifax.rwth-aachen.de/archlinux/iso/2018.12.01/archlinux-2018.12.01-x86_64.iso',
     'https://ftp.halifax.rwth-aachen.de/archlinux/iso/2018.12.01/archlinux-2018.12.01-x86_64.iso.sig',
     'https://ftp.halifax.rwth-aachen.de/archlinux/iso/2018.12.01/archlinux-2018.12.01-x86_64.iso.torr',
     'https://ftp.halifax.rwth-aachen.de/archlinux/iso/2018.12.01/archlinux-bootstrap-2018.12.01-x86_64.tar.gz',
     'https://ftp.halifax.rwth-aachen.de/archlinux/iso/2018.12.01/archlinux-bootstrap-2018.12.01-x86_64.tar.gz.sig',
     'https://ftp.halifax.rwth-aachen.de/archlinux/iso/2018.12.01/md5sums.txt',
     'https://ftp.halifax.rwth-aachen.de/archlinux/iso/2018.12.01/sha1sums.txt']

    Will fail if the request fails. If parsing fails, return an empty list.
    Optionally, only include links in the list which return something ok on
    HTTP HEAD (might take a while).
    """
    filelike_p = re.compile(r"[0-9-_\w.]*[.][a-z0-9]{2,4}")

    # Find base url to prepend on relative links.
    pr = urlparse(url)
    pr = pr._replace(path=os.path.dirname(pr.path))
    pr = pr._replace(params="")
    pr = pr._replace(query="")
    pr = pr._replace(fragment="")
    baseurl = pr.geturl()

    r = requests.get(url)
    if r.status_code >= 400:
        raise RuntimeError("fetch failed with %s: %s", r.status_code, url)

    soup = bs4.BeautifulSoup(r.text, features="lxml")
    links = set()

    for a in soup.find_all("a"):
        match = filelike_p.search(a.get("href"))
        if not match:
            continue

        # This might be an absolute link or just a filename.
        link = match.group()
        if not link.startswith("http") and not link.startswith("/"):
            link = os.path.join(baseurl, link)
        if not with_head or requests.head(link).status_code < 400:
            links.add(link)

    return sorted(links)

def compare_files(a, b):
    """
    Compare two paths by checksum. Returns True, if files are byte-identical.
    """
    with open(a) as fa:
        chka = hashlib.sha1()
        while True:
            data = fa.read(4096)
            if not data:
                break
            chka.update(data)

    with open(b) as fb:
        chkb = hashlib.sha1()
        while True:
            data = fb.read(4096)
            if not data:
                break
            chkb.update(data)

    return chka.hexdigest() == chkb.hexdigest()

def xmlstream(filename, tag):
    """
    Given a path to an XML file and a tag name (without namespace), stream
    through the XML, and emit the element denoted by tag for processing, e.g.
    via xmltodict, more convenient DOM parser or something else.

        for snippet in xmlstream("sample.xml", "sometag"):
            print(len(snippet))

    """
    def strip_ns(tag):
        if not '}' in tag:
            return tag
        return tag.split('}')[1]

    # https://stackoverflow.com/a/13261805, http://effbot.org/elementtree/iterparse.htm
    context = iter(ET.iterparse(filename, events=('start', 'end',)))
    _, root = next(context)

    for event, elem in context:
        if not strip_ns(elem.tag) == tag or event == 'start':
            continue

        yield ET.tostring(elem)
        root.clear()

